@@ -3,7 +3,7 @@
 '''
 A library to access Tryton's models like a client.
 '''
-__version__ = "2.4.4"
+__version__ = "2.6.0"
 __all__ = ['Model', 'Wizard']
 import sys
 try:
@@ -31,7 +31,7 @@ class _EvalEnvironment(dict):
         self.parent = parent
 
     def __getitem__(self, item):
-        if item == '_parent_' + self.parent._parent_name \
+        if item == '_parent_' + self.parent._parent_field_name \
                 and self.parent.parent:
             return _EvalEnvironment(self.parent.parent)
         return self.parent._get_eval()[item]
@@ -214,12 +214,17 @@ class One2ManyDescriptor(FieldDescriptor):
     default = []
 
     def __get__(self, instance, owner):
+        from .pyson import PYSONDecoder
         relation = Model.get(self.definition['relation'], instance._config)
         value = super(One2ManyDescriptor, self).__get__(instance, owner)
         if not isinstance(value, ModelList):
-            value = ModelList((relation(id)
-                for id in value or []), instance, self.name,
-                self.definition.get('relation_field', ''))
+            ctx = instance._context.copy() if instance._context else {}
+            if self.definition.get('context'):
+                decoder = PYSONDecoder(_EvalEnvironment(instance))
+                ctx.update(decoder.decode(self.definition.get('context')))
+            with instance._config.set_context(ctx):
+                value = ModelList(self.definition, (relation(id)
+                        for id in value or []), instance, self.name)
             instance._values[self.name] = value
         return value
 
@@ -246,13 +251,13 @@ class ReferenceValueDescriptor(ValueDescriptor):
         value = super(ReferenceValueDescriptor, self).__get__(instance, owner)
         if isinstance(value, Model):
             value = '%s,%s' % (value.__class__.__name__, value.id)
-        return value or False
+        return value or None
 
 
 class Many2OneValueDescriptor(ValueDescriptor):
     def __get__(self, instance, owner):
         value = super(Many2OneValueDescriptor, self).__get__(instance, owner)
-        return value and value.id or False
+        return value and value.id or None
 
 
 class One2OneValueDescriptor(Many2OneValueDescriptor):
@@ -263,18 +268,14 @@ class One2ManyValueDescriptor(ValueDescriptor):
     def __get__(self, instance, owner):
         value = [('add', [])]
         value_list = getattr(instance, self.name)
-        parent_name = self.definition.get('relation_field', '')
         for record in value_list:
-            if record.id >= 0:
-                values = record._get_values(fields=record._changed)
-                values.pop(parent_name, None)
-                if record._changed and values:
-                    value.append(('write', record.id, values))
+            if record.id > 0:
+                if record._changed:
+                    value.append(('write', [record.id], record._get_values(
+                        fields=record._changed)))
                 value[0][1].append(record.id)
             else:
-                values = record._get_values()
-                values.pop(parent_name, None)
-                value.append(('create', values))
+                value.append(('create', record._get_values()))
         if value_list.record_removed:
             value.append(('unlink', [x.id for x in value_list.record_removed]))
         if value_list.record_deleted:
@@ -301,7 +302,7 @@ class ReferenceEvalDescriptor(EvalDescriptor):
         value = super(ReferenceEvalDescriptor, self).__get__(instance, owner)
         if isinstance(value, Model):
             value = '%s,%s' % (value.__class__.__name__, value.id)
-        return value or False
+        return value or None
 
 
 class Many2OneEvalDescriptor(EvalDescriptor):
@@ -309,7 +310,7 @@ class Many2OneEvalDescriptor(EvalDescriptor):
         value = super(Many2OneEvalDescriptor, self).__get__(instance, owner)
         if value:
             return value.id
-        return False
+        return None
 
 
 class One2OneEvalDescriptor(Many2OneEvalDescriptor):
@@ -408,22 +409,26 @@ class MetaModelFactory(object):
 class ModelList(list):
     'List for Model'
 
-    def __init__(self, sequence=None, parent=None, parent_field_name='',
-            parent_name=''):
+    def __init__(self, definition, sequence=None, parent=None,
+            parent_field_name=''):
+        self.model_name = definition['relation']
         if sequence is None:
             sequence = []
         self.parent = parent
         if parent:
             assert parent_field_name
         self.parent_field_name = parent_field_name
-        self.parent_name = parent_name
+        self.parent_name = definition.get('relation_field', '')
+        self.domain = definition.get('domain', [])
+        self.context = definition.get('context')
+        self.add_remove = definition.get('add_remove')
         self.record_removed = set()
         self.record_deleted = set()
         result = super(ModelList, self).__init__(sequence)
         for record in self:
             record._parent = parent
             record._parent_field_name = parent_field_name
-            record._parent_name = parent_name
+            record._parent_name = self.parent_name
         return result
     __init__.__doc__ = list.__init__.__doc__
 
@@ -433,6 +438,13 @@ class ModelList(list):
             self.parent._changed.add(self.parent_field_name)
             self.parent._on_change(self.parent_field_name)
 
+    def _get_context(self):
+        from .pyson import PYSONDecoder
+        decoder = PYSONDecoder(_EvalEnvironment(self.parent))
+        ctx = self.parent._context.copy() if self.parent._context else {}
+        ctx.update(decoder.decode(self.context) if self.context else {})
+        return ctx
+
     def append(self, record):
         assert isinstance(record, Model)
         if self.parent:
@@ -482,8 +494,8 @@ class ModelList(list):
     def pop(self, index=-1):
         self.record_removed.add(self[index])
         self[index]._parent = None
-        self[index]._parent_field_name = ''
-        self[index]._parent_name = ''
+        self[index]._parent_field_name = None
+        self[index]._parent_name = None
         res = super(ModelList, self).pop(index)
         self._changed()
         return res
@@ -492,8 +504,8 @@ class ModelList(list):
     def remove(self, record):
         self.record_deleted.add(record)
         record._parent = None
-        record._parent_field_name = ''
-        record._parent_name = ''
+        record._parent_field_name = None
+        record._parent_name = None
         res = super(ModelList, self).remove(record)
         self._changed()
         return res
@@ -507,6 +519,28 @@ class ModelList(list):
         raise NotImplementedError
     sort.__doc__ = list.sort.__doc__
 
+    def new(self):
+        'Adds a new record to the ModelList and returns it'
+        Relation = Model.get(self.model_name, self.parent._config)
+        with Relation._config.set_context(self._get_context()):
+            new_record = Relation()
+        self.append(new_record)
+        return new_record
+
+    def find(self, condition=None, offset=0, limit=None, order=None):
+        'Returns records matching condition taking into account list domain'
+        from .pyson import PYSONDecoder
+        decoder = PYSONDecoder(_EvalEnvironment(self.parent))
+        Relation = Model.get(self.model_name, self.parent._config)
+        if condition is None:
+            condition = []
+        field_domain = decoder.decode(self.domain)
+        add_remove_domain = (decoder.decode(self.add_remove)
+            if self.add_remove else [])
+        new_domain = [field_domain, add_remove_domain, condition]
+        with Relation._config.set_context(self._get_context()):
+            return Relation.find(new_domain, offset, limit, order)
+
 
 class Model(object):
     'Model class for Tryton records'
@@ -528,6 +562,7 @@ class Model(object):
         self._parent = None  # store the parent record
         self._parent_field_name = ''  # store the field name in parent record
         self._parent_name = ''  # store the field name to parent record
+        self._context = self._config.context  # store the context
         if self.id < 0 and _default:
             self._default_get()
 
@@ -551,9 +586,6 @@ class Model(object):
     @classmethod
     def get(cls, name, config=None):
         'Get a class for the named Model'
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-
         class Spam(Model):
             __metaclass__ = MetaModelFactory(name, config=config)()
         return Spam
@@ -620,7 +652,7 @@ class Model(object):
                 return
             values = self._get_values(fields=self._changed)
             context['_timestamp'] = self._get_timestamp()
-            self._proxy.write(self.id, values, context)
+            self._proxy.write([self.id], values, context)
         self.reload()
 
     def delete(self):
@@ -628,7 +660,7 @@ class Model(object):
         if self.id > 0:
             context = self._config.context
             context['_timestamp'] = self._get_timestamp()
-            return self._proxy.delete(self.id, context)
+            return self._proxy.delete([self.id], context)
         self.reload()
         return True
 
@@ -667,8 +699,8 @@ class Model(object):
                     if y['loading'] == 'eager']
         if not self._fields:
             fields.append('_timestamp')
-        self._values.update(self._proxy.read(self.id, fields,
-            self._config.context))
+        self._values.update(self._proxy.read([self.id], fields,
+            self._config.context)[0])
         for field in fields:
             if (field in self._fields
                     and self._fields[field]['type'] == 'float'
@@ -697,8 +729,8 @@ class Model(object):
                     record = relation()
                     record._default_set(vals)
                     records.append(record)
-                self._values[field] = ModelList(records, self, field,
-                        definition.get('relation_field', ''))
+                self._values[field] = ModelList(definition, records, self,
+                    field)
             else:
                 self._values[field] = value
 
@@ -710,10 +742,10 @@ class Model(object):
 
     def _get_on_change_value(self):
         values = self._get_eval()
-        del values['id']
         for field, definition in self._fields.iteritems():
-            if definition['type'] in ('one2many', 'many2many'):
-                values[field] = [x._get_eval() for x in getattr(self, field)]
+            if definition['type'] == 'one2many':
+                values[field] = [x._get_on_change_value()
+                    for x in getattr(self, field)]
         return values
 
     def _on_change_args(self, args):
@@ -808,8 +840,8 @@ class Model(object):
             values.update(self._on_change_args(definition['on_change_with']))
         if to_change:
             context = self._config.context
-            result = getattr(self._proxy, 'on_change_with')(list(to_change),
-                values, context)
+            result = getattr(self._proxy, 'on_change_with')(values,
+                list(to_change), context)
             for field, value in result.items():
                 self._on_change_set(field, value)
         for field in later:
@@ -847,7 +879,7 @@ class Wizard(object):
         assert state in self.states
 
         self.state = state
-        while self.state != self.end_state:
+        while self.state != 'end':
             ctx = self._context.copy()
             ctx.update(self._config.context)
             if self.models:
@@ -860,7 +892,11 @@ class Wizard(object):
                 ctx['active_model'] = None
 
             if self.form:
-                data = {self.form_state: self.form._get_on_change_value()}
+                # Filter only modified values
+                data = {self.form_state:
+                    dict((k, v) for k, v in
+                        self.form._get_on_change_value().iteritems()
+                        if k in self.form._values)}
             else:
                 data = {}
 
